/*
 * Copyright (c) 1999-2021 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */

#ifndef PARADISO_CONTEXT_HPP
#define PARADISO_CONTEXT_HPP

#include "geometry.hpp"
#include "rgba.hpp"


#include <memory>

namespace paradiso {

struct Context final {

    Context();
    ~Context();

    Context& set_blend();
    Context& set_depth();

    Context& set_viewport(const Rectangle& v);
    Rectangle viewport() const;

    Size viewport_size() const;
    Point viewport_origin() const;

    Context& set_clearcolor(const RGBA &clear_color);
    RGBA clearcolor() const;

    void clear();

    uint32_t get_error() const;

    struct impl;

private:
    std::unique_ptr<impl> impl_;
};

}

#endif
